# django imports
from django.conf import settings
from django.contrib.flatpages.models import FlatPage
from django.test import TestCase

# reviews imports
from portlets.example.models import TextPortlet
from portlets.models import PortletAssignment
from portlets.models import PortletBlocking
from portlets.models import PortletRegistration
from portlets.models import Slot
import portlets.utils


settings.INSTALLED_APPS += ("portlets.example", )


class PortletsModelsTestCase(TestCase):
    """Tests the models
    """
    def test_portlet(self):
        """
        """
        text_portlet = TextPortlet.objects.create(title="Text")
        self.assertEqual(text_portlet.title, "Text")
        self.failIf(text_portlet.render().find("portlet-header") == -1)
        self.failIf(text_portlet.form().as_p().find('id="id_title"') == -1)

    def test_slot(self):
        """
        """
        slot = Slot.objects.create(name="Left")
        self.assertEqual(slot.name, "Left")

    def test_portlet_registration(self):
        """
        """
        portlet_registration = PortletRegistration.objects.create(
            type="textportlet",
            name="TextPortlet",
        )

        self.assertEqual(portlet_registration.type, "textportlet")
        self.assertEqual(portlet_registration.name, "TextPortlet")
        self.assertEqual(portlet_registration.active, True)

        # add another portlet with other name
        PortletRegistration.objects.create(
            type="textportlet_2",
            name="TextPortlet_2",
        )

    def test_portlet_assignment(self):
        """
        """
        slot = Slot.objects.create(name="Left")
        page = FlatPage.objects.create(url="/test/", title="Test")
        portlet = TextPortlet.objects.create(title="Text")

        portlet_assignment = PortletAssignment.objects.create(
            slot=slot,
            content=page,
            portlet=portlet,
        )

        self.assertEqual(portlet_assignment.slot, slot)
        self.assertEqual(portlet_assignment.content, page)
        self.assertEqual(portlet_assignment.portlet, portlet)

    def test_portlet_blocking(self):
        """
        """
        slot = Slot.objects.create(name="Left")
        page = FlatPage.objects.create(url="/test/", title="Test")

        portlet_blocking = PortletBlocking.objects.create(
            slot=slot,
            content=page,
        )

        self.assertEqual(portlet_blocking.slot, slot)
        self.assertEqual(portlet_blocking.content, page)


class PortletsUtilsTestCase(TestCase):
    """Tests the methods of utils.py
    """
    def setUp(self):
        """
        """
        # Create a page which server as content
        self.page = FlatPage.objects.create(url="/test/", title="Test")

        # Create a portlet
        self.portlet = TextPortlet.objects.create(title="Text")

        # Create some slots
        self.left_slot = Slot.objects.create(name="Left")
        self.right_slot = Slot.objects.create(name="Right")

    def test_get_slots(self):
        """
        """
        # We need to register the portlet first
        portlets.utils.register_portlet(TextPortlet, "TextPortlet")

        # At the beginning no slot has portlets
        slots = portlets.utils.get_slots(self.page)
        self.assertEqual(slots["has_portlets"], False)
        self.assertEqual(slots["items"][0]["name"], "Left")
        self.assertEqual(slots["items"][0]["portlets"], [])
        self.assertEqual(slots["items"][1]["name"], "Right")
        self.assertEqual(slots["items"][1]["portlets"], [])

        # Assigning the text portlet to the left slot of the page
        PortletAssignment.objects.create(
            slot=self.left_slot, content=self.page, portlet=self.portlet, position=1)

        slots = portlets.utils.get_slots(self.page)
        self.assertEqual(slots["has_portlets"], True)
        self.assertEqual(slots["items"][0]["name"], "Left")
        self.assertEqual(slots["items"][0]["portlets"][0]["title"], "Text")
        self.assertEqual(slots["items"][1]["name"], "Right")
        self.assertEqual(slots["items"][1]["portlets"], [])

        # Create another portlet ...
        portlet_2 = TextPortlet.objects.create(title="Text 2")

        # ... and assign it also to the left slot of the page
        PortletAssignment.objects.create(
            slot=self.left_slot, content=self.page, portlet=portlet_2, position=2)

        slots = portlets.utils.get_slots(self.page)
        self.assertEqual(slots["has_portlets"], True)
        self.assertEqual(slots["items"][0]["name"], "Left")
        self.assertEqual(slots["items"][0]["portlets"][0]["title"], "Text")
        self.assertEqual(slots["items"][0]["portlets"][1]["title"], "Text 2")
        self.assertEqual(slots["items"][1]["name"], "Right")
        self.assertEqual(slots["items"][1]["portlets"], [])

        # Create another portlet ...
        portlet_3 = TextPortlet.objects.create(title="Text 3")

        # ... and assign it the right slot of the page
        PortletAssignment.objects.create(
            slot=self.right_slot, content=self.page, portlet=portlet_3, position=1)

        slots = portlets.utils.get_slots(self.page)
        self.assertEqual(slots["has_portlets"], True)
        self.assertEqual(slots["items"][0]["name"], "Left")
        self.assertEqual(slots["items"][0]["portlets"][0]["title"], "Text")
        self.assertEqual(slots["items"][0]["portlets"][1]["title"], "Text 2")
        self.assertEqual(slots["items"][1]["name"], "Right")
        self.assertEqual(slots["items"][1]["portlets"][0]["title"], "Text 3")

    def test_has_portlets(self):
        """
        """
        # At the beginning no slot has portlets
        result = portlets.utils.has_portlets(self.page, self.left_slot)
        self.assertEqual(result, False)

        result = portlets.utils.has_portlets(self.page, self.right_slot)
        self.assertEqual(result, False)

        # Assigning the text portlet to the left slot of the page
        PortletAssignment.objects.create(
            slot=self.left_slot, content=self.page, portlet=self.portlet, position=1)

        result = portlets.utils.has_portlets(self.page, self.left_slot)
        self.assertEqual(result, True)

        result = portlets.utils.has_portlets(self.page, self.right_slot)
        self.assertEqual(result, False)

        # Assigning the text portlet to the right slot of the page
        PortletAssignment.objects.create(
            slot=self.right_slot, content=self.page, portlet=self.portlet, position=1)

        result = portlets.utils.has_portlets(self.page, self.left_slot)
        self.assertEqual(result, True)

        result = portlets.utils.has_portlets(self.page, self.right_slot)
        self.assertEqual(result, True)

    def test_has_portlets_2(self):
        """Uses methods.
        """
        # At the beginning no slot has portlets
        result = self.left_slot.has_portlets(self.page)
        self.assertEqual(result, False)

        result = self.right_slot.has_portlets(self.page)
        self.assertEqual(result, False)

        # Assigning the text portlet to the left slot of the page
        PortletAssignment.objects.create(
            slot=self.left_slot, content=self.page, portlet=self.portlet, position=1)

        result = self.left_slot.has_portlets(self.page)
        self.assertEqual(result, True)

        result = self.right_slot.has_portlets(self.page)
        self.assertEqual(result, False)

        # Assigning the text portlet to the right slot of the page
        PortletAssignment.objects.create(
            slot=self.right_slot, content=self.page, portlet=self.portlet, position=1)

        result = self.left_slot.has_portlets(self.page)
        self.assertEqual(result, True)

        result = self.right_slot.has_portlets(self.page)
        self.assertEqual(result, True)

    def test_register_portlets(self):
        """
        """
        # At the beginning no portlets are registered
        result = portlets.utils.get_registered_portlets()
        self.assertEqual(result, {})

        # Register the TextPortlet
        portlets.utils.register_portlet(TextPortlet, "TextPortlet")
        result = portlets.utils.get_registered_portlets()
        self.assertEqual(result, {u"textportlet": u"TextPortlet"})

        # Unregister the TextPortlet
        portlets.utils.unregister_portlet(TextPortlet)
        result = portlets.utils.get_registered_portlets()
        self.assertEqual(result, {})

    def test_is_blocked(self):
        """
        """
        # Assigning the text portlet to the left slot of the page
        PortletAssignment.objects.create(
            slot=self.left_slot, content=self.page, portlet=self.portlet, position=1)

        # Assigning the text portlet to the left slot of the page
        PortletAssignment.objects.create(
            slot=self.right_slot, content=self.page, portlet=self.portlet, position=1)

        result = portlets.utils.is_blocked(self.page, self.left_slot)
        self.assertEqual(result, False)

        result = portlets.utils.is_blocked(self.page, self.right_slot)
        self.assertEqual(result, False)

        # Blocking the left slot of the page
        PortletBlocking.objects.create(slot=self.left_slot, content=self.page)

        result = portlets.utils.is_blocked(self.page, self.left_slot)
        self.assertEqual(result, True)

        result = portlets.utils.is_blocked(self.page, self.right_slot)
        self.assertEqual(result, False)

        # Blocking the left right of the page
        PortletBlocking.objects.create(slot=self.right_slot, content=self.page)

        result = portlets.utils.is_blocked(self.page, self.left_slot)
        self.assertEqual(result, True)

        result = portlets.utils.is_blocked(self.page, self.right_slot)
        self.assertEqual(result, True)

    def test_is_blocked_2(self):
        """Uses methods."""
        # Assigning the text portlet to the left slot of the page
        PortletAssignment.objects.create(
            slot=self.left_slot, content=self.page, portlet=self.portlet, position=1)

        # Assigning the text portlet to the left slot of the page
        PortletAssignment.objects.create(
            slot=self.right_slot, content=self.page, portlet=self.portlet, position=1)

        result = self.left_slot.is_blocked(self.page)
        self.assertEqual(result, False)

        result = self.right_slot.is_blocked(self.page)
        self.assertEqual(result, False)

        # Blocking the left slot of the page
        PortletBlocking.objects.create(slot=self.left_slot, content=self.page)

        result = self.left_slot.is_blocked(self.page)
        self.assertEqual(result, True)

        result = self.right_slot.is_blocked(self.page)
        self.assertEqual(result, False)

        # Blocking the left right of the page
        PortletBlocking.objects.create(slot=self.right_slot, content=self.page)

        result = self.left_slot.is_blocked(self.page)
        self.assertEqual(result, True)

        result = self.right_slot.is_blocked(self.page)
        self.assertEqual(result, True)

    def test_get_portlets(self):
        """Uses methods.
        """
        # At the beginning no slot has portlets
        result = portlets.utils.get_portlets(self.page, self.left_slot)
        self.assertEqual(result, [])

        result = portlets.utils.get_portlets(self.page, self.right_slot)
        self.assertEqual(result, [])

        # Assigning the text portlet to the left slot of the page
        PortletAssignment.objects.create(
            slot=self.left_slot, content=self.page, portlet=self.portlet, position=1)

        result = portlets.utils.get_portlets(self.page, self.left_slot)
        self.assertEqual(result, [self.portlet])

        result = portlets.utils.get_portlets(self.page, self.right_slot)
        self.assertEqual(result, [])

        # Assigning the text portlet to the right slot of the page
        PortletAssignment.objects.create(
            slot=self.right_slot, content=self.page, portlet=self.portlet, position=1)

        result = portlets.utils.get_portlets(self.page, self.left_slot)
        self.assertEqual(result, [self.portlet])

        result = portlets.utils.get_portlets(self.page, self.right_slot)
        self.assertEqual(result, [self.portlet])

    def test_get_portlets_2(self):
        """Uses methods.
        """
        # At the beginning no slot has portlets
        result = self.left_slot.get_portlets(self.page)
        self.assertEqual(result, [])

        result = self.right_slot.get_portlets(self.page)
        self.assertEqual(result, [])

        # Assigning the text portlet to the left slot of the page
        PortletAssignment.objects.create(
            slot=self.left_slot, content=self.page, portlet=self.portlet, position=1)

        result = self.left_slot.get_portlets(self.page)
        self.assertEqual(result, [self.portlet])

        result = self.right_slot.get_portlets(self.page)
        self.assertEqual(result, [])

        # Assigning the text portlet to the right slot of the page
        PortletAssignment.objects.create(
            slot=self.right_slot, content=self.page, portlet=self.portlet, position=1)

        result = self.left_slot.get_portlets(self.page)
        self.assertEqual(result, [self.portlet])

        result = self.right_slot.get_portlets(self.page)
        self.assertEqual(result, [self.portlet])



# portlets imports
import portlets.utils


class PortletsBase(object):
    """Mixin class to make objects portlets aware.
    """
    def get_portlets(self, slot):
        """Returns True if the passed slot is blocked. Otherwise False.

        **Parameters:**

            slot
                The slot for which the blocking is tested. Must be a Slot
                instance.
        """
        return slot.get_portlets(self)

    def get_slots(self):
        """Returns all slots with all assigned portlets.
        """
        return portlets.utils.get_slots(self)

    def has_portlets(self, slot):
        """Returns True if the ther are portlets for the passed slot.

        **Parameters:**

            slot
                The slot which is tested. Must be a Slot instance.
        """
        return slot.has_portlets(self)

    def is_blocked(self, slot):
        """Returns True if the passed slot is blocked. Otherwise False.

        **Parameters:**

            slot
                The slot for which the blocking is tested. Must be a Slot
                instance.
        """
        return slot.is_blocked(self)

    def get_parent_for_portlets(self):
        """Returns the parent from which portlets are inherited.
        """
        return None



# django imports
from django.contrib.contenttypes.models import ContentType
from django.db.models import Q

# portlets imports
from portlets.models import PortletAssignment
from portlets.models import PortletRegistration
from portlets.models import Slot

# Load logger
import logging
logger = logging.getLogger(__name__)


def get_slots(obj):
    """Returns all slots with all assigned portlets for the passed object.

    **Parameters:**

        obj
            The obj for which the slots should be returned.
    """
    portlet_types = get_registered_portlets()
    ct = ContentType.objects.get_for_model(obj)

    # Stores whether at least one slot has at least one portlet.
    has_portlets = False

    items = []
    for slot in Slot.objects.all():
        temp = []
        for pa in PortletAssignment.objects.filter(
            slot=slot, content_id=obj.id, content_type=ct.id):
            has_portlets = True

            # Display only registered portlets
            portlet_type = portlet_types.get(pa.portlet.__class__.__name__.lower())
            if portlet_type:
                temp.append({
                    "pa_id": pa.id,
                    "title": pa.portlet.title,
                    "type": portlet_types.get(pa.portlet.__class__.__name__.lower(), ""),
                })

        items.append({
            "id": slot.id,
            "name": slot.name,
            "is_blocked": slot.is_blocked(obj),
            "portlets": temp,
        })

    return {
        "has_portlets": has_portlets,
        "items": items
    }


# DEPRECATED 1.2
def is_blocked(obj, slot):
    """
    Returns True if the passed slot is blocked for the passed object.
    Otherwise False.

    **Parameters:**

        obj
            The object for which the blocking is tested. Must be a Django
            model instance.

        slot
            The slot for which the blocking is tested. Must be a Slot
            instance.
    """
    logger.info("Decprecated: portlets: the utility function 'is_blocked' is deprecated. Please use 'Slot.is_blocked'.")
    return slot.is_blocked(obj)


# DEPRECATED 1.2
def has_portlets(obj, slot):
    """Returns True if the passed object has portlets for passed slot.

    **Parameters:**

        obj
            The object which is tested. Must be a Django model instance.

        slot
            The slot which is tested. Must be a Slot instance.
    """
    logger.info("Decprecated: portlets: the utility function 'has_portlets' is deprecated. Please use 'Slot.has_portlets'.")
    return slot.has_portlets(obj)


# DEPRECATED 1.2
def get_portlets(obj, slot):
    """Returns portlet objs for a given slot and obj (content object).

    **Parameters**

    slot
        The slot for which the portlets should be returned. Must be a Slot
        instance.

    obj
        The object for the portlets should be returned. Must be a Django model
        instance.

    """
    logger.info("Decprecated: portlets: the utility function 'get_portlets' is deprecated. Please use 'Slot.get_portlets'.")
    return slot.get_portlets(obj)


def get_registered_portlets():
    """Returns registered portlet types as dict.
    """
    portlet_types = {}
    for pr in PortletRegistration.objects.all():
        portlet_types[pr.type] = pr.name

    return portlet_types


def register_portlet(klass, name):
    """Registers a portlet. Name and klass must both be unique.

    **Parameters**

    klass
        The portlet's python class

    name
        Then unique name under which the portlet is registered
    """
    type = klass.__name__.lower()
    if not PortletRegistration.objects.filter(Q(type=type) | Q(name=name)):
        PortletRegistration.objects.create(type=type, name=name)


def unregister_portlet(klass):
    """Unregisters portlet the passed portlet.

    **Parameters**

    klass
        The portlet's python class
    """
    type = klass.__name__.lower()
    try:
        pr = PortletRegistration.objects.get(type=type)
    except PortletRegistration.DoesNotExist:
        pass
    else:
        pr.delete()



# django imports
from django import template
from django.conf import settings
from django.core.cache import cache

# portlets imports
from portlets.models import Slot

register = template.Library()


@register.inclusion_tag('portlets/portlet_slot.html', takes_context=True)
def portlet_slot(context, slot_name, instance=None):
    """
    Returns the portlets for given slot and instance. If the instance implements
    the ``get_parent_for_portlets`` method the portlets of the parent of the
    instance are also added.
    """
    request = context.get("request")

    # CACHE
    content_type = instance.__class__.__name__.lower()
    cache_key = "%s-portlets-%s-%s-%s-%s-%s" % (settings.CACHE_MIDDLEWARE_KEY_PREFIX,
        content_type, instance.id, slot_name, request.user.id, context.get("CURRENT_LANGUAGE"))

    rendered_portlets = cache.get(cache_key)

    if rendered_portlets:
        return {"portlets": rendered_portlets}

    if instance is None:
        return {"portlets": []}

    try:
        slot = Slot.objects.get(name=slot_name)
    except Slot.DoesNotExist:
        return {"portlets": []}

    # Get portlets for given instance
    temp = slot.get_portlets(instance)

    # Get inherited portlets
    try:
        instance.get_parent_for_portlets()
    except AttributeError:
        instance = None

    while instance:
        # If the portlets are blocked no portlets should be added
        if slot.is_blocked(instance):
            break

        # If the instance has no get_parent_for_portlets, there are no portlets
        try:
            instance = instance.get_parent_for_portlets()
        except AttributeError:
            break

        # If there is no parent for portlets, there are no portlets to add
        if instance is None:
            break

        parent_portlets = slot.get_portlets(instance)
        parent_portlets.reverse()
        for p in parent_portlets:
            if p not in temp:
                temp.insert(0, p)

    rendered_portlets = []
    for portlet in temp:
        rendered_portlets.append(portlet.render(context))

    cache.set(cache_key, rendered_portlets)

    return {"portlets": rendered_portlets}



# django imports
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.db import models
from django.utils.translation import ugettext_lazy as _


class Portlet(models.Model):
    """Base portlet. All portlets should inherit from this class.
    """
    title = models.CharField(_(u"Title"), blank=True, max_length=100)

    class Meta:
        abstract = True

    def render(self):
        """Renders the portlet as html. Have to be implemented by sub classes.
        """
        raise NotImplemented

    def form(self, **kwargs):
        """returns the form for the portlet. Have to be implemented by sub
        classes.
        """
        raise NotImplemented


class PortletAssignment(models.Model):
    """Assigns portlets to slots and content.
    """
    slot = models.ForeignKey("Slot", verbose_name=_(u"Slot"))

    content_type = models.ForeignKey(ContentType, related_name="pa_content")
    content_id = models.PositiveIntegerField()
    content = generic.GenericForeignKey('content_type', 'content_id')

    portlet_type = models.ForeignKey(ContentType, related_name="pa_portlets")
    portlet_id = models.PositiveIntegerField()
    portlet = generic.GenericForeignKey('portlet_type', 'portlet_id')

    position = models.PositiveSmallIntegerField(_("Position"), default=999)

    class Meta:
        ordering = ["position"]
        verbose_name_plural = _(u"Portlet assignments")

    def __unicode__(self):
        try:
            return "%s (%s)" % (self.portlet.title, self.slot.name)
        except AttributeError:
            return ""


class PortletBlocking(models.Model):
    """Blocks portlets for given slot and content object.
    """
    slot = models.ForeignKey("Slot", verbose_name=_(u"Slot"))

    content_type = models.ForeignKey(ContentType, related_name="pb_content")
    content_id = models.PositiveIntegerField()
    content = generic.GenericForeignKey('content_type', 'content_id')

    class Meta:
        unique_together = ["slot", "content_id", "content_type"]

    def __unicode__(self):
        try:
            return "%s (%s)" % (self.content.title, self.slot.name)
        except AttributeError:
            return ""


class PortletRegistration(models.Model):
    """
    Registered portlets. These are provided to be added to customer.

    Parameters:

        * type:
            The type of the portlet. This must be the exactly class name of
            the portlet, e.g. TextPortlet

        * name
            The name of the portlet. This is displayed to the user.

        * active
            If true the portlet will be provided to assign to content object
     """
    type = models.CharField(_(u"Type"), max_length=30, unique=True)
    name = models.CharField(_(u"Name"), max_length=50, unique=True)
    active = models.BooleanField(_(u"Active"), default=True)

    class Meta:
        ordering = ("name", )

    def __unicode__(self):
        return "%s %s" % (self.type, self.name)


class Slot(models.Model):
    """
    Slots are places to which portlets can be assigned.
    """
    name = models.CharField(_("Name"), max_length=50)

    def __unicode__(self):
        return self.name

    def get_portlets(self, obj):
        """
        Returns portlet objs for a given slot and obj (content object).

        **Parameters**

        obj
            The object for the portlets should be returned. Must be a Django model
            instance.
        """
        ctype = ContentType.objects.get_for_model(obj)
        portlet_assignments = PortletAssignment.objects.filter(
            slot=self, content_type=ctype.id, content_id=obj.id).order_by("position")

        portlets = []
        for portlet_assignment in portlet_assignments:
            portlets.append(portlet_assignment.portlet)

        return portlets

    def has_portlets(self, obj):
        """
        Returns True if the passed object has portlets for passed slot.

        **Parameters:**

            obj
                The object which is tested. Must be a Django model instance.
        """
        while obj:
            if len(self.get_portlets(obj)) > 0:
                return True
            if self.is_blocked(obj):
                break
            try:
                obj = obj.get_parent_for_portlets()
            except AttributeError:
                break

        return False

    def is_blocked(self, obj):
        """
        Returns True if the passed slot is blocked for the passed object.
        Otherwise False.

        **Parameters:**

            slot
                The slot for which the blocking is tested. Must be a Slot
                instance.
        """
        ct = ContentType.objects.get_for_model(obj)
        try:
            PortletBlocking.objects.get(slot=self, content_type=ct.id, content_id=obj.id)
        except PortletBlocking.DoesNotExist:
            return False
        else:
            return True
